/**
 * Display a slider to adjust the search filter depth.
 *
 *
 * ### Examples
 * Apply .remotesearchUiSearchFilterSlider() to input field with id 'ProductSearchField'.
 *
 * JavaScript:
 *
 *    $('.ep-searchfilter-facetsearch-startdepth').remotesearchUiSearchFilterSlider({
 *      steps: 4,
 *      value: 2,
 *      minSelectable: 1
 *    });
 *
 * HTML:
 *
 *    <div class="ep-searchfilter-facetsearch-startdepth"><!-- entry point for jQuery widget *remotesearchUiSearchFilterSlider* --></div>
 *
 *
 * @class jQuery.ui.remotesearchUiSearchFilterSlider
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses de_epages
 * @uses jQuery.tmpl
 * @uses jQuery.ui.slider
 * @uses ep.ui.slider
 * @uses jQuery.ui.widget
 * @since 7.68.0
 */

/**
 * @cfg {integer} [steps] An integer who defines the number of steps for the slider (default = 4).
 */

/**
 * @cfg {integer} [value] A start value for the slider (default = 2).
 */

/**
 * @cfg {number} [min] An integer value of minimum value of the slider (default = 0).
 */

/**
 * @cfg {number} [minSelectable] An integer value of minimum selectable position of the slider (default = 0).
 */

/**
 * @cfg {Object} [rangeColor] An object with properties for the slider range color (background and highlight).
 */

/**
 * See `jQuery.ui.remotesearchUiSearchFilterSlider` for details.
 *
 * @param {integer} [steps] An integer who defines the number of steps for the slider (default = 4).
 * @param {integer} [value] A start value for the slider (default = 2).
 * @param {number} [min] An integer value of minimum value of the slider (default = 0).
 * @param {number} [minSelectable] An integer value of minimum selectable position of the slider (default = 0).
 * @param {Object} [rangeColor] An object with properties for the slider range color (background and highlight).
 *
 * @method remotesearchUiSearchFilterSlider
 * @member jQuery
 *
 * @since 7.68.0
 */

/**
 * Builds the UI for the slider from a javascript template.
 *
 * @method _buildUI
 * @member jQuery.ui.remotesearchUiSearchFilterSlider
 *
 * @since 7.68.0
 */

/**
 * Configures and initializes the slider to adjust the search filter depth.
 *
 * @method _initSlider
 * @member jQuery.ui.remotesearchUiSearchFilterSlider
 *
 * @since 7.68.0
 */

/**
 * Builds the scale for the slider.
 *
 * @method _buildScale
 * @member jQuery.ui.remotesearchUiSearchFilterSlider
 *
 * @since 7.68.0
 */

/**
 * Magic method to set widget options.
 *
 * @method _setOption
 * @member jQuery.ui.remotesearchUiSearchFilterSlider
 *
 * @since 7.68.0
 */

/**
 * Do all stuff needed if a new value is set.
 *
 * @method _setNewValue
 * @member jQuery.ui.remotesearchUiSearchFilterSlider
 *
 * @since 7.70.0
 */

/**
 * Function to handle resize event.
 *
 * @method _resize
 * @member jQuery.ui.remotesearchUiSearchFilterSlider
 *
 * @since 7.68.0
 */

/**
 * Creates the gradient color value for the CSS background property,
 * depending on the current slider step.
 *
 * @method _createLinearGradient
 * @member jQuery.ui.remotesearchUiSearchFilterSlider
 *
 * @since 7.68.0
 */

/*
 * @copyright       © Copyright 2006-2022, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages.remotesearch.ui.searchfilter-slider
 */

define("de_epages/remotesearch/ui/searchfilter-slider", [
    "jquery",
    "ep",
    "de_epages",
    "$dict!../dictionary",
    "$tmpl!./searchfilter-slider",

    "jquery/ui/slider",
    "jquery/metaparse",
    "ep/ui/slider",
    "ep/ui/input",
    "jquery/ui/widget",
    "jquery/tmpl"
], function ($, ep, de_epages, remotesearchDict, tmplSearchFilter) {

/*
 *
 * @dictionary          de_epages.remotesearch.dictionary
 *
 * @translation         {ExampleBreadcrumbHome}
 *                      {ExampleBreadcrumbLevel1}
 *                      {ExampleBreadcrumbLevel2}
 *                      {ExampleBreadcrumbLevel3}
 *                      {ExampleBreadcrumbLevel4}
 *                      {SearchFilterStartDepthFromHome}
 *                      {SearchFilterStartDepthFromSub}
 */

    var translations = {
            startPage: remotesearchDict.translate("StartPage"),
            category: remotesearchDict.translate("Category"),
            startDepthFromHome: remotesearchDict.translate("SearchFilterStartDepthFromHome"),
            startDepthFromSub: remotesearchDict.translate("SearchFilterStartDepthFromSub"),
            exampleBreadcrumbHome: remotesearchDict.translate("ExampleBreadcrumbHome"),
            exampleBreadcrumbLevel1: remotesearchDict.translate("ExampleBreadcrumbLevel1"),
            exampleBreadcrumbLevel2: remotesearchDict.translate("ExampleBreadcrumbLevel2"),
            exampleBreadcrumbLevel3: remotesearchDict.translate("ExampleBreadcrumbLevel3"),
            exampleBreadcrumbLevel4: remotesearchDict.translate("ExampleBreadcrumbLevel4")
        };

    $.widget( "ui.remotesearchUiSearchFilterSlider", {
    //  slider,
    //  value,
        dom: {},
        hResizeTimer: undefined,

        options: {
            steps: 4,
            value: 2,
            min: 0,
            minSelectable: 0,
            rangeColor: {
                background: "rgba(142, 142, 142, 1)",
                highlight: "rgba(0, 212, 255, 1)"
            }
        },

        _create: function(){
            this._superApply(arguments);

            var self = this,
                o = self.options;

            self.value = o.value;

            // recalculate slider width, when no resize event has been fired for 150 ms
            $(window).on('resize.remotesearchUiSearchFilterSlider', $.proxy(self, '_resize'));

            self._buildUI();
        },

        /*
         * Function to handle resize event.
         */
        _resize: function() {
            var self = this;

            clearTimeout(self.hResizeTimer);
            self.hResizeTimer = setTimeout($.proxy(self, '_buildScale'), 150);
        },

        /*
         * Creates the UI.
         */
        _buildUI: function() {
            var self = this,
                elem = self.element,
                o = self.options,

                renderedTmpl;

            // render view template for basic template
            renderedTmpl = tmplSearchFilter({
                translations: translations,
                steps: o.steps,
                facetedSearchStartDepth: self.value
            });

            // load template and push named tmpl elements to current instance
            $.extend(self.dom, renderedTmpl.tmplItem('elements'));
            // append template
            elem.append(renderedTmpl);
            // metaparser
            elem
                .find('.ep-js')
                .metaparse();

            // build UI elements
            self._initSlider();
            self._buildScale();

            // due to Firefox' render engine we need to wait until DOM is ready
            window.setTimeout(function() {
                self._resize();
            }, 800);
        },

        /*
         * Ininitalizes the slider and configures the handling when sliding.
         */
        _initSlider: function() {
            var self = this,
                elem = self.element,
                o = self.options;

            if (!self.slider) {
                // initialize slider
                self.slider = self.dom.sliderElem.slider({
                    value: self.value,
                    min: o.min,
                    max: o.steps,
                    step: 1,
                    classes: {
                        "ui-slider": "ep-searchfilter-facetsearch-startdepth-slider-element",
                        "ui-slider-handle": "fa fa-caret-up",
                        "ui-slider-range": "ep-searchfilter-facetsearch-startdepth-slider-range"
                    },
                    slide: function(event, ui) {
                        if (Number(ui.value) < o.minSelectable) return false;

                        var background = self._createLinearGradient(Number(ui.value));

                        self.slider.css({
                            background: background
                        });

                        // update info text in template and add/remove highlight class for first scale-dot
                        self.dom.infoContent.html(translations.startDepthFromSub);
                        self.dom.scaleList.find('li:eq(0) .scale-dot').removeClass('scale-dot-highlight');

                    },
                    stop: function(event, ui) {
                        if (Number(ui.value) < o.minSelectable) return false;

                        var form = self.dom.facetedSearchStartDepth[0].form,
                            $form = ep(form);

                        self.value = Number(ui.value);

                        // update hidden input and establish onbefore handling for the form
                        $form.length && $form.uiForm({showSaveWarn: true});
                        self.dom.facetedSearchStartDepth.val(ui.value);
                        self.dom.facetedSearchStartDepth.trigger('changeAttr', {value: ui.value});
                        $form.length && $form.trigger('change');
                    },
                    create: function(event, ui) {
                        var background = self._createLinearGradient(self.value);

                        self.dom.sliderElem.css({
                            background: background
                        });
                    }
                });
            }
        },

        /*
         * Creates a scale for the slider and set values for the template the UI.
         */
        _buildScale: function() {
            var self = this,
                o = self.options,
                sliderWidth = self.dom.sliderElem.outerWidth(),
                part = sliderWidth / o.steps,
                outerWidth = 0;

            // set width for scale parts
            self.dom.scaleList.children('li').each(function() {
                var li = $(this);

                li.css({
                    width: part + "px"
                });

                outerWidth += part;
            });

            // calculate the necessary left padding for the scale container
            // (correct indentation for first label)
            self.dom.scaleContainer.css({
                "padding-left": (self.dom.firstLabelContent.outerWidth() / 2) + "px"
            });

            // calculate the width of the complete scale and its holder element
            self.dom.scaleList.css({
                width: outerWidth + "px",
                minWidth: outerWidth + "px"
            });
            self.dom.scaleHolder.css({
                width: sliderWidth + "px",
                minWidth: sliderWidth + "px"
            });
        },

        /*
         * Creates the gradient color value for the CSS background property,
         * depending on the current slider step.
         */
        _createLinearGradient: function(val) {
            var self = this,
                o = self.options,

                percent = 100 * (val - o.min) / (o.steps - o.min);

            return "linear-gradient(90deg, " + o.rangeColor.background + " 0%, " + o.rangeColor.background + " " + percent + "%, " + o.rangeColor.highlight + " " + percent + "%, " + o.rangeColor.highlight + " 100%)";
        },

        /*
         * Magic method to set widget options.
         */
        _setOption: function (key, value) {
            var self = this,
                o = self.options;

            if (key === 'value') {
                // if new value is lower than the minimal selectable step
                if (value < o.minSelectable) {
                    value = o.minSelectable;
                }

                self._setNewValue(value);
            } else if (key === 'minSelectable') {
                // if current value is lower than the new minimal selectable step
                if (self.value < value) {
                    self._setNewValue(value);
                }
            }

            return this._superApply(arguments);
        },

        /*
         * Do all stuff needed if a new value is set.
         */
        _setNewValue: function (value) {
            var self = this;

            // save new value
            self.value = value;
            // set new value
            self.slider.slider('value', value);
            // triggern relevant slider events
            self.slider.slider('instance')._trigger('slide', event, {value: value});
            self.slider.slider('instance')._trigger('stop', event, {value: value});
        },

        /*
         * Destroy widget.
         */
        destroy: function() {
            var self = this,
                elem = self.element;

            // remove event handler
            $(window).off('resize.remotesearchUiSearchFilterSlider');
            self.slider.slider('destroy');
            self.dom.facetedSearchStartDepth.uiInput('destroy');

            // clean up html
            elem.empty();

            // do inherited stuff
            self._superApply(arguments);
        }

    });

    return de_epages;
});
